unit Server.Data.Main;

interface

uses
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Error, FireDAC.UI.Intf, FireDAC.Phys.Intf, FireDAC.Stan.Def,
  FireDAC.Stan.Pool, FireDAC.Stan.Async, FireDAC.Phys, FireDAC.Phys.SQLite,
  FireDAC.Phys.SQLiteDef, FireDAC.Stan.ExprFuncs, FireDAC.VCLUI.Wait, Data.DB,
  FireDAC.Comp.Client, FireDAC.Stan.Param, FireDAC.DatS, FireDAC.DApt.Intf,
  FireDAC.DApt, FireDAC.Comp.DataSet;

type
  TUserInfo = record
    Roles: TArray<string>;
    Audience: TArray<string>;

    class function New: TUserInfo; static;
  end;

  /// <summary>
  ///   Auth DataModule
  /// </summary>
  TdmMain = class(TDataModule)
    conMain: TFDConnection;
  private
    { Private declarations }
  public
    function GetUserRoles(const AUser, APassword: string): TArray<string>;
    function GetUserInfo(const AUser, APassword: string): TUserInfo;
  end;

var
  dmMain: TdmMain;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

{ TdmMain }

function TdmMain.GetUserInfo(const AUser, APassword: string): TUserInfo;
const
  SQL = 'SELECT * FROM USERS WHERE (USERNAME=:USER) AND (PASSWORD=:PASS)';
var
  LQuery: TFDQuery;
begin
  Result := TUserInfo.New;
  LQuery := TFDQuery.Create(nil);
  LQuery.Connection := conMain;
  try
    LQuery.SQL.Text := SQL;
    LQuery.ParamByName('USER').AsString := AUser;
    LQuery.ParamByName('PASS').AsString := APassword;
    LQuery.Open;
    if not LQuery.IsEmpty then
    begin
      Result.Roles := LQuery.FieldByName('Roles').AsString.Split([',']);
      Result.Audience := LQuery.FieldByName('Audience').AsString.Split([',']);
    end;
  finally
    LQuery.Free;
  end;
end;

function TdmMain.GetUserRoles(const AUser, APassword: string): TArray<string>;
const
  SQL = 'SELECT ROLES FROM USERS WHERE (USERNAME=:USER) AND (PASSWORD=:PASS)';
var
  LQuery: TFDQuery;
begin
  Result := [];
  LQuery := TFDQuery.Create(nil);
  LQuery.Connection := conMain;
  try
    LQuery.SQL.Text := SQL;
    LQuery.ParamByName('USER').AsString := AUser;
    LQuery.ParamByName('PASS').AsString := APassword;
    LQuery.Open;
    if not LQuery.IsEmpty then
      Result := LQuery.FieldByName('Roles').AsString.Split([',']);
  finally
    LQuery.Free;
  end;
end;

{ TUserInfo }

class function TUserInfo.New: TUserInfo;
begin
  Result.Roles := [];
  Result.Audience := [];
end;

end.
