unit Server.Resources;

{$I WiRL.inc}

interface

uses
  System.Classes, System.SysUtils, System.Rtti,

  WiRL.Core.JSON,
  WiRL.Core.Application,
  WiRL.Core.Registry,
  WiRL.Core.Attributes,
  WiRL.http.Accept.MediaType,
  WiRL.http.URL,
  WiRL.Core.MessageBody.Default,
  WiRL.http.Request,
  WiRL.http.Response,
  WiRL.Core.Auth.Context,
  WiRL.Core.Auth.Resource,

  Server.Auth.Resource;

type
  [Path('basic_auth')]
  TBasicAuthResource = class(TWiRLAuthBasicResource)
  private
    // Injects the custom claims into "Subject" field
    [Context] Subject: TWiRLSubject;
  protected
    function Authenticate(const AUserName, APassword: string): TWiRLAuthResult; override;
  end;

implementation

uses
  Server.Data.Main,
  System.DateUtils,
  REST.Json;

{ TBasicAuthResource }

function TBasicAuthResource.Authenticate(const AUserName, APassword: string): TWiRLAuthResult;
var
  LUser: TUserInfo;
begin
  LUser := dmMain.GetUserInfo(AUserName, APassword);

  Result.Roles := LUser.Roles;
  Result.Success := Length(Result.Roles) > 0;

  // Here you can set all field of your custom claims object
  Subject.Issuer := 'identity_server';
  Subject.AudienceArray := LUser.Audience;
  Subject.Expiration := IncMinute(Now, 20);
end;

initialization
  TWiRLResourceRegistry.Instance.RegisterResource<TBasicAuthResource>;

end.
